@@ -3,7 +3,13 @@ require 'mkmf'
 sdl_config = with_config('sdl-config', 'sdl-config')
 
 $CFLAGS += ' ' + `#{sdl_config} --cflags`.chomp
-$LDFLAGS += ' ' + `#{sdl_config} --libs`.chomp
+$LOCAL_LIBS += ' ' + `#{sdl_config} --libs`.chomp
+
+have_func("TTF_OpenFontIndex","SDL_ttf.h")
+have_func("TTF_FontFaces","SDL_ttf.h")
+have_func("TTF_FontFaceIsFixedWidth","SDL_ttf.h")
+have_func("TTF_FontFaceFamilyName","SDL_ttf.h")
+have_func("TTF_FontFaceStyleName","SDL_ttf.h")
 
 if enable_config("event2",true) then
   $CFLAGS+= " -D DEF_EVENT2"
@@ -24,6 +30,9 @@ end
 if have_library("smpeg","SMPEG_new") then
   $CFLAGS+= " -D HAVE_SMPEG "
 end
+if have_library("SDLSKK","SDLSKK_Context_new") then
+  $CFLAGS+= " -D HAVE_SDLSKK "
+end
 if enable_config("opengl",false) then
   dir_config('x11','/usr/X11R6')
   
