@@ -1,7 +1,7 @@
 /*
   Ruby/SDL   Ruby extension library for SDL
 
-  Copyright (C) 2001 Ohbayashi Ippei
+  Copyright (C) 2001,2002 Ohbayashi Ippei
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -24,12 +24,12 @@
 static VALUE sdl_load(VALUE class,VALUE filename)
 {
   SDL_Surface *surface;
-  surface = IMG_Load(STR2CSTR(filename));
+  surface = IMG_Load(GETCSTR(filename));
   if(surface==NULL){
-    rb_raise(eSDLError,"Couldn't load %s: %s",STR2CSTR(filename),
+    rb_raise(eSDLError,"Couldn't load %s: %s",GETCSTR(filename),
 	     SDL_GetError());
   }
-  return Data_Wrap_Struct(class,0,SDL_FreeSurface,surface);
+  return Data_Wrap_Struct(class,0,sdl_freeSurface,surface);
 }
 
 void init_sdl_image()
