@@ -1,7 +1,7 @@
 /*
   Ruby/SDL   Ruby extension library for SDL
 
-  Copyright (C) 2001 Ohbayashi Ippei
+  Copyright (C) 2001,2002 Ohbayashi Ippei
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -56,6 +56,7 @@ void init_ttf();
 #endif
 #ifdef HAVE_SDL_MIXER
 void  init_mixer();
+void quit_mixer();
 #endif
 #ifdef HAVE_SMPEG
 void init_smpeg();
@@ -70,7 +71,6 @@ static VALUE sdl_init(VALUE obj,VALUE flags)
   flag= NUM2UINT(flags);
   if( SDL_Init(flag) < 0 )
     rb_raise(eSDLError,"Couldn't initialize SDL: %s",SDL_GetError());
-  atexit(sdl_quit);
   return Qnil;
 }
 
@@ -79,6 +79,11 @@ static VALUE sdl_wasInit(VALUE mod,VALUE flags)
   return UINT2NUM( SDL_WasInit(NUM2UINT(flags)) );
 }
 
+static int is_quit=0;
+int rubysdl_is_quit(void)
+{
+  return is_quit;
+}
 static void sdl_quit()
 {
 #ifdef HAVE_SDL_MIXER
@@ -87,6 +92,7 @@ static void sdl_quit()
 #ifdef HAVE_SDL_TTF
   quit_ttf();
 #endif
+  is_quit = 1;
   SDL_Quit();
   return ;
 }
@@ -147,7 +153,10 @@ void Init_sdl()
 #ifdef HAVE_SMPEG
   init_smpeg();
 #endif
-  
+#ifdef HAVE_SDLSKK
+  init_sdlskk();
+#endif
+  rb_set_end_proc(sdl_quit,0); 
 }
 
 
