@@ -1,7 +1,7 @@
 /*
   Ruby/SDL   Ruby extension library for SDL
 
-  Copyright (C) 2001 Ohbayashi Ippei
+  Copyright (C) 2001,2002 Ohbayashi Ippei
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -19,6 +19,13 @@
   */
 #include "rubysdl.h"
 
+void sdl_freeSurface(SDL_Surface* surface)
+{
+  if( !rubysdl_is_quit() ){
+    SDL_FreeSurface(surface);
+  }
+}
+
 Uint32 VALUE2COLOR(VALUE color,SDL_PixelFormat *format)
 {
   if( rb_obj_is_kind_of( color, rb_cArray ) ){
@@ -44,6 +51,13 @@ static VALUE sdl_getVideoSurface(VALUE mod)
   return Data_Wrap_Struct(cSurface,0,0,surface);
 }
 
+static VALUE sdl_videoDriverName(VALUE mod)
+{
+  char namebuf[256];
+  SDL_VideoDriverName( namebuf, sizeof(namebuf) );
+  return rb_str_new2( namebuf );
+}
+
 static VALUE sdl_listModes(VALUE mod,VALUE flags)
 {
   SDL_Rect **modes;
@@ -78,6 +92,8 @@ static VALUE sdl_getVideoInfo(VALUE mod)
   const SDL_VideoInfo *info;
   VALUE obj;
   info = SDL_GetVideoInfo();
+  if(info==NULL)
+    rb_raise(eSDLError,"Couldn't get video information");
   obj=rb_obj_alloc(cVideoInfo);
   rb_iv_set(obj,"@hw_available",BOOL(info->hw_available));
   rb_iv_set(obj,"@wm_available",BOOL(info->wm_available));
@@ -90,9 +106,6 @@ static VALUE sdl_getVideoInfo(VALUE mod)
   rb_iv_set(obj,"@blit_fill",BOOL(info->blit_fill));
   rb_iv_set(obj,"@video_mem",UINT2NUM(info->video_mem));
   rb_iv_set(obj,"@bpp",UINT2NUM(info->vfmt->BitsPerPixel));
-#if 0
-  rb_iv_set(obj,"@vfmt",Data_Wrap_Struct(cPixelFormat,0,0,info->vfmt));
-#endif
   return obj;
 }
 
@@ -115,6 +128,16 @@ static VALUE sdl_flip(VALUE obj)
   return Qnil;
 }
 
+static VALUE sdl_toggleFullScreen(VALUE obj)
+{
+  SDL_Surface *screen;
+  Data_Get_Struct(obj,SDL_Surface,screen);
+  if( SDL_WM_ToggleFullScreen(screen)==0 ){
+    rb_raise( eSDLError,"toggle full screen fail : %s", SDL_GetError() );
+  }
+  return Qnil;
+}
+
 static VALUE sdl_setVideoMode(VALUE mod,VALUE w,VALUE h,VALUE bpp,
        VALUE flags)
 {
@@ -125,11 +148,8 @@ static VALUE sdl_setVideoMode(VALUE mod,VALUE w,VALUE h,VALUE bpp,
   if( screen==NULL ){
     rb_raise(eSDLError,"Cound't set %dx%d %d bpp video mode: %s",
 	     NUM2INT(w),NUM2INT(h),NUM2INT(bpp),SDL_GetError());
-  }
-  screenObject = Data_Wrap_Struct(cSurface,0,0,screen);
-  rb_define_singleton_method(screenObject,"updateRect",sdl_updateRect,4);
-  rb_define_singleton_method(screenObject,"flip",sdl_flip,0);
-  return screenObject;
+  }  
+  return Data_Wrap_Struct(cScreen,0,0,screen);
 }
 static VALUE sdl_setGamma(VALUE mod,VALUE rgamma,VALUE ggamma,VALUE bgamma)
 {
@@ -138,6 +158,45 @@ static VALUE sdl_setGamma(VALUE mod,VALUE rgamma,VALUE ggamma,VALUE bgamma)
   return Qnil;
 }
 
+static VALUE sdl_getGammaRamp(VALUE mod)
+{
+  Uint16 table[3][256];
+  VALUE ary_table, ary_subtable;
+  int i,j;
+  
+  if( SDL_GetGammaRamp( table[0], table[1], table[2] ) == -1 ){
+    rb_raise(eSDLError,"cannot get gamma lookup table: %s",SDL_GetError());
+  }
+  
+  ary_table = rb_ary_new();
+  for( i=0; i<3; ++i ){
+    ary_subtable = rb_ary_new();
+    for( j=0; j<256; ++j ){
+      rb_ary_push( ary_subtable, INT2FIX(table[i][j]) );
+    }
+    rb_ary_push( ary_table, ary_subtable );
+  }
+  return ary_table;
+}
+
+static VALUE sdl_setGammaRamp(VALUE mod, VALUE ary_table)
+{
+  Uint16 table[3][256];
+  VALUE ary_subtable;
+  int i,j;
+
+  for( i=0; i<3; ++i ){
+    ary_subtable = rb_ary_entry( ary_table, i );
+    for( j=0; j<256; ++j ){
+      table[i][j] = NUM2INT( rb_ary_entry( ary_subtable, j ) );
+    }
+  }
+  if( SDL_SetGammaRamp( table[0], table[1], table[2] )==-1 ){
+    rb_raise(eSDLError,"cannot set gamma lookup table: %s",SDL_GetError());
+  }
+  return Qnil;
+}
+
 static VALUE sdl_createSurface(VALUE class,VALUE flags,VALUE w,VALUE h,
 			       VALUE format)
 {
@@ -157,20 +216,29 @@ static VALUE sdl_createSurface(VALUE class,VALUE flags,VALUE w,VALUE h,
   if( newSurface==NULL ){
     rb_raise( eSDLError,"Couldn't Create Surface: %s",SDL_GetError() );
   }
-  return Data_Wrap_Struct(class,0,SDL_FreeSurface,newSurface);
+  return Data_Wrap_Struct(class,0,sdl_freeSurface,newSurface);
 }
 				    
 static VALUE sdl_loadBMP(VALUE class,VALUE filename)
 {
   SDL_Surface *image;
-  image=SDL_LoadBMP(STR2CSTR(filename));
+  image=SDL_LoadBMP(GETCSTR(filename));
   if( image==NULL ){
     rb_raise(eSDLError,"Couldn't Load BMP file %s : %s",
-	     STR2CSTR(filename),SDL_GetError());
+	     GETCSTR(filename),SDL_GetError());
   }
-  return Data_Wrap_Struct(class,0,SDL_FreeSurface,image);
+  return Data_Wrap_Struct(class,0,sdl_freeSurface,image);
 }
-
+static VALUE sdl_saveBMP(VALUE obj,VALUE filename)
+{
+  SDL_Surface *image;
+  Data_Get_Struct( obj, SDL_Surface, image );
+  if( SDL_SaveBMP( image, GETCSTR(filename) )==-1 ){
+    rb_raise(eSDLError,"cannot save %s: %s",GETCSTR(filename),SDL_GetError());
+  }
+  return Qnil;
+}
+  
 static VALUE sdl_displayFormat(VALUE obj)
 {
   SDL_Surface *srcImage,*destImage;
@@ -179,7 +247,18 @@ static VALUE sdl_displayFormat(VALUE obj)
   if( destImage==NULL ){
     rb_raise(eSDLError,"Couldn't convert surface format: %s",SDL_GetError());
   }
-  return Data_Wrap_Struct(cSurface,0,SDL_FreeSurface,destImage);
+  return Data_Wrap_Struct(cSurface,0,sdl_freeSurface,destImage);
+}
+
+static VALUE sdl_displayFormatAlpha(VALUE obj)
+{
+  SDL_Surface *srcImage,*destImage;
+  Data_Get_Struct(obj,SDL_Surface,srcImage);
+  destImage=SDL_DisplayFormatAlpha(srcImage);
+  if( destImage==NULL ){
+    rb_raise(eSDLError,"Couldn't convert surface format: %s",SDL_GetError());
+  }
+  return Data_Wrap_Struct(cSurface,0,sdl_freeSurface,destImage);
 }
 
 static VALUE sdl_setColorKey(VALUE obj,VALUE flag,VALUE key)
@@ -250,6 +329,24 @@ static VALUE sdl_setClipRect(VALUE obj,VALUE x,VALUE y,VALUE w,VALUE h)
   SDL_SetClipRect(surface,&rect);
 }
 
+static VALUE sdl_getClipRect(VALUE obj)
+{
+  SDL_Surface *surface;
+  SDL_Rect rect;
+  VALUE ary_rect;
+  
+  Data_Get_Struct( obj, SDL_Surface, surface );
+  SDL_GetClipRect( surface, &rect );
+  
+  ary_rect = rb_ary_new();
+  rb_ary_push( ary_rect, INT2FIX(rect.x) );
+  rb_ary_push( ary_rect, INT2FIX(rect.y) );
+  rb_ary_push( ary_rect, INT2FIX(rect.w) );
+  rb_ary_push( ary_rect, INT2FIX(rect.h) );
+  
+  return ary_rect;
+}
+
 static VALUE sdl_fillRect(VALUE obj,VALUE x,VALUE y,VALUE w,VALUE h,
 		      VALUE color)
 {
@@ -462,11 +559,14 @@ static void defineConstForVideo()
 void init_video()
 {
   rb_define_module_function(mSDL,"getVideoSurface",sdl_getVideoSurface,0);
+  rb_define_module_function(mSDL,"videoDriverName",sdl_videoDriverName,0);
   rb_define_module_function(mSDL,"blitSurface",sdl_blitSurface,8);
   rb_define_module_function(mSDL,"setVideoMode",sdl_setVideoMode,4);
   rb_define_module_function(mSDL,"checkVideoMode",sdl_checkVideoMode,4);
   rb_define_module_function(mSDL,"listModes",sdl_listModes,1);
   rb_define_module_function(mSDL,"setGamma",sdl_setGamma,3);
+  rb_define_module_function(mSDL,"getGammaRamp",sdl_getGammaRamp,0);
+  rb_define_module_function(mSDL,"setGammaRamp",sdl_setGammaRamp,1);
   cVideoInfo=rb_define_class_under(mSDL,"VideoInfo",rb_cObject);
   rb_define_attr(cVideoInfo,"hw_available",1,0);
   rb_define_attr(cVideoInfo,"wm_available",1,0);
@@ -479,9 +579,7 @@ void init_video()
   rb_define_attr(cVideoInfo,"blit_fill",1,0);
   rb_define_attr(cVideoInfo,"video_mem",1,0);
   rb_define_attr(cVideoInfo,"bpp",1,0);
-#if 0
-  rb_define_attr(cVideoInfo,"vfmt",1,0);
-#endif
+
   
   rb_define_module_function(mSDL,"videoInfo",sdl_getVideoInfo,0);
   
@@ -489,10 +587,13 @@ void init_video()
 
   rb_define_singleton_method(cSurface,"new",sdl_createSurface,4);
   rb_define_singleton_method(cSurface,"loadBMP",sdl_loadBMP,1);
+  rb_define_method(cSurface,"saveBMP",sdl_saveBMP,1);
   rb_define_method(cSurface,"displayFormat",sdl_displayFormat,0);
+  rb_define_method(cSurface,"displayFormatAlpha",sdl_displayFormatAlpha,0);
   rb_define_method(cSurface,"setColorKey",sdl_setColorKey,2);
   rb_define_method(cSurface,"fillRect",sdl_fillRect,5);
   rb_define_method(cSurface,"setClipRect",sdl_setClipRect,4);
+  rb_define_method(cSurface,"getClipRect",sdl_getClipRect,0);
   rb_define_method(cSurface,"setAlpha",sdl_setAlpha,2);
   rb_define_method(cSurface,"h",sdl_surfaceH,0);
   rb_define_method(cSurface,"w",sdl_surfaceW,0);
@@ -514,6 +615,10 @@ void init_video()
   rb_define_method(cSurface,"colorkey",sdl_getColorkey,0);
   rb_define_method(cSurface,"alpha",sdl_getAlpha,0);
 
+  cScreen = rb_define_class_under(mSDL,"Screen",cSurface);
+  rb_define_method(cScreen,"updateRect",sdl_updateRect,4);
+  rb_define_method(cScreen,"flip",sdl_flip,0);
+  rb_define_method(cScreen,"toggleFullScreen",sdl_toggleFullScreen,0);
   defineConstForVideo();
   return;
 }
