@@ -7,22 +7,19 @@ SDL::WM::setCaption($0,$0)
 
 SDL::TTF.init
 
-font = SDL::TTF.open('font.ttf',24)
+font = SDL::TTF.open('nihongo.ttf',24)
 font.style = SDL::TTF::STYLE_NORMAL
 
 font.drawSolidUTF8(screen,'test SDL_ttf',300,150,255,255,255)
 
 screen.flip
 
-event=SDL::Event.new
-
-while true 
-  if  event.poll != 0 then
-    if event.type==SDL::Event::QUIT then
-      break
-    end
-    if event.type==SDL::Event::KEYDOWN then
+while true
+  while event = SDL::Event2.poll
+    case event
+    when SDL::Event2::KeyDown, SDL::Event2::Quit
       exit
     end
   end
+  
 end
