@@ -1,10 +1,12 @@
 require 'sdl'
 
 SDL.init( SDL::INIT_VIDEO|SDL::INIT_AUDIO )
+SDL::Mixer.open
 
 screen = SDL.setVideoMode( 320, 240, 16, SDL::SWSURFACE )
 
 mpeg = SDL::MPEG.load( 'sample.mpg' )
+
 info = mpeg.info
 
 p(info)
@@ -14,7 +16,6 @@ mpeg.enableVideo true
 
 mpeg.setDisplay(screen)
 mpeg.setDisplayRegion( 0, 0, screen.w, screen.h )
-
 mpeg.play
 
 loop do
@@ -36,6 +37,11 @@ loop do
     end
   end
 
+  if mpeg.status != SDL::MPEG::PLAYING then
+    mpeg.stop
+    exit
+  end
+      
   sleep 0.1
   
 end
