@@ -1,6 +1,7 @@
 require 'sdl'
 
-SDL::init(SDL::INIT_AUDIO)
+SDL::init(SDL::INIT_AUDIO|SDL::INIT_VIDEO)
+SDL.setVideoMode(640,480,16,SDL::SWSURFACE)
 
 SDL::Mixer.open(22050,SDL::Mixer::FORMAT_U8,2,4096)
 wave=SDL::Mixer::Wave.load("sample.wav")
