@@ -15,15 +15,13 @@ screen.drawFilledCircle(300,300,30,Red)
 
 screen.flip
 
-event=SDL::Event.new
-
-while true 
-  if  event.poll != 0 then
-    if event.type==SDL::Event::QUIT then
-      break
-    end
-    if event.type==SDL::Event::KEYDOWN then
+while true
+  while event = SDL::Event2.poll
+    case event
+    when SDL::Event2::KeyDown, SDL::Event2::Quit
       exit
     end
   end
+
+  sleep 0.2
 end
